@@ -6,11 +6,12 @@ import json
 import os
 import sys
 import textwrap
+from importlib import import_module  # buffalo buffalo
 
 from .util import six
 
 from . import Collection, Config, Executor, FilesystemLoader
-from .complete import complete
+from .completion.complete import complete, print_completion_script
 from .parser import Parser, ParserContext, Argument
 from .exceptions import UnexpectedExit, CollectionNotFound, ParseError, Exit
 from .terminals import pty_size
@@ -39,12 +40,21 @@ class Program(object):
         """
         # Arguments present always, even when wrapped as a different binary
         return [
+            Argument(
+                names=("command-timeout", "T"),
+                kind=int,
+                help="Specify a global command execution timeout, in seconds.",
+            ),
             Argument(
                 names=("complete",),
                 kind=bool,
                 default=False,
                 help="Print tab-completion candidates for given parse remainder.",  # noqa
             ),
+            Argument(
+                names=("config", "f"),
+                help="Runtime configuration file to use.",
+            ),
             Argument(
                 names=("debug", "d"),
                 kind=bool,
@@ -52,16 +62,10 @@ class Program(object):
                 help="Enable debug output.",
             ),
             Argument(
-                names=("prompt-for-sudo-password",),
+                names=("dry", "R"),
                 kind=bool,
                 default=False,
-                help="Prompt user at start of session for the sudo.password config value.",  # noqa
-            ),
-            Argument(
-                names=("write-pyc",),
-                kind=bool,
-                default=False,
-                help="Enable creation of .pyc files.",
+                help="Echo commands instead of running.",
             ),
             Argument(
                 names=("echo", "e"),
@@ -69,10 +73,6 @@ class Program(object):
                 default=False,
                 help="Echo executed commands before running.",
             ),
-            Argument(
-                names=("config", "f"),
-                help="Runtime configuration file to use.",
-            ),
             Argument(
                 names=("help", "h"),
                 optional=True,
@@ -98,6 +98,18 @@ class Program(object):
                 help="Change the display format used when listing tasks. Should be one of: flat (default), nested, json.",  # noqa
                 default="flat",
             ),
+            Argument(
+                names=("print-completion-script",),
+                kind=str,
+                default="",
+                help="Print the tab-completion script for your preferred shell (bash|zsh|fish).",  # noqa
+            ),
+            Argument(
+                names=("prompt-for-sudo-password",),
+                kind=bool,
+                default=False,
+                help="Prompt user at start of session for the sudo.password config value.",  # noqa
+            ),
             Argument(
                 names=("pty", "p"),
                 kind=bool,
@@ -116,6 +128,12 @@ class Program(object):
                 default=False,
                 help="Warn, instead of failing, when shell commands fail.",
             ),
+            Argument(
+                names=("write-pyc",),
+                kind=bool,
+                default=False,
+                help="Enable creation of .pyc files.",
+            ),
         ]
 
     def task_args(self):
@@ -164,6 +182,7 @@ class Program(object):
         loader_class=None,
         executor_class=None,
         config_class=None,
+        binary_names=None,
     ):
         """
         Create a new, parameterized `.Program` instance.
@@ -193,15 +212,30 @@ class Program(object):
             binstub installed as ``foobar``, it will default to ``Foobar``.
 
         :param str binary:
-            The binary name as displayed in ``--help`` output.
+            Descriptive lowercase binary name string used in help text.
+
+            For example, Invoke's own internal value for this is ``inv[oke]``,
+            denoting that it is installed as both ``inv`` and ``invoke``. As
+            this is purely text intended for help display, it may be in any
+            format you wish, though it should match whatever you've put into
+            your ``setup.py``'s ``console_scripts`` entry.
 
             If ``None`` (default), uses the first word in ``argv`` verbatim (as
             with ``name`` above, except not capitalized).
 
-            Giving this explicitly may be useful when you install your program
-            under multiple names, such as Invoke itself does - it installs as
-            both ``inv`` and ``invoke``, and sets ``binary="inv[oke]"`` so its
-            ``--help`` output implies both names.
+        :param binary_names:
+            List of binary name strings, for use in completion scripts.
+
+            This list ensures that the shell completion scripts generated by
+            :option:`--print-completion-script` instruct the shell to use
+            that completion for all of this program's installed names.
+
+            For example, Invoke's internal default for this is ``["inv",
+            "invoke"]``.
+
+            If ``None`` (the default), the first word in ``argv`` (in the
+            invocation of :option:`--print-completion-script`) is used in a
+            single-item list.
 
         :param loader_class:
             The `.Loader` subclass to use when loading task collections.
@@ -211,17 +245,26 @@ class Program(object):
         :param executor_class:
             The `.Executor` subclass to use when executing tasks.
 
-            Defaults to `.Executor`.
+            Defaults to `.Executor`; may also be overridden at runtime by the
+            :ref:`configuration system <default-values>` and its
+            ``tasks.executor_class`` setting (anytime that setting is not
+            ``None``).
 
         :param config_class:
             The `.Config` subclass to use for the base config object.
 
             Defaults to `.Config`.
+
+        .. versionchanged:: 1.2
+            Added the ``binary_names`` argument.
         """
         self.version = "unknown" if version is None else version
         self.namespace = namespace
         self._name = name
+        # TODO 2.0: rename binary to binary_help_name or similar. (Or write
+        # code to autogenerate it from binary_names.)
         self._binary = binary
+        self._binary_names = binary_names
         self.argv = None
         self.loader_class = loader_class or FilesystemLoader
         self.executor_class = executor_class or Executor
@@ -270,16 +313,22 @@ class Program(object):
             run["hide"] = self.args.hide.value
         if self.args.echo.value:
             run["echo"] = True
+        if self.args.dry.value:
+            run["dry"] = True
         tasks = {}
         if "no-dedupe" in self.args and self.args["no-dedupe"].value:
             tasks["dedupe"] = False
+        timeouts = {}
+        command = self.args["command-timeout"].value
+        if command:
+            timeouts["command"] = command
         # Handle "fill in config values at start of runtime", which for now is
         # just sudo password
         sudo = {}
         if self.args["prompt-for-sudo-password"].value:
             prompt = "Desired 'sudo.password' config value: "
             sudo["password"] = getpass.getpass(prompt)
-        overrides = dict(run=run, tasks=tasks, sudo=sudo)
+        overrides = dict(run=run, tasks=tasks, sudo=sudo, timeouts=timeouts)
         self.config.load_overrides(overrides, merge=False)
         runtime_path = self.args.config.value
         if runtime_path is None:
@@ -374,12 +423,20 @@ class Program(object):
         if self.args.debug.value:
             enable_logging()
 
-        # Print version & exit if necessary
+        # Short-circuit if --version
         if self.args.version.value:
             debug("Saw --version, printing version & exiting")
             self.print_version()
             raise Exit
 
+        # Print (dynamic, no tasks required) completion script if requested
+        if self.args["print-completion-script"].value:
+            print_completion_script(
+                shell=self.args["print-completion-script"].value,
+                names=self.binary_names,
+            )
+            raise Exit
+
     def parse_collection(self):
         """
         Load a tasks collection & project-level config.
@@ -422,7 +479,7 @@ class Program(object):
 
         .. versionadded:: 1.0
         """
-        halp = self.args.help.value or self.core_via_tasks.args.help.value
+        halp = self.args.help.value
 
         # Core (no value given) --help output (only when bundled namespace)
         if halp is True:
@@ -461,7 +518,12 @@ class Program(object):
 
         # Print completion helpers if necessary
         if self.args.complete.value:
-            complete(self.core, self.initial_context, self.collection)
+            complete(
+                names=self.binary_names,
+                core=self.core,
+                initial_context=self.initial_context,
+                collection=self.collection,
+            )
 
         # Fallback behavior if no tasks were given & no default specified
         # (mostly a subroutine for overriding purposes)
@@ -483,11 +545,24 @@ class Program(object):
 
         .. note::
             Client code just wanting a different `.Executor` subclass can just
-            set ``executor_class`` in `.__init__`.
+            set ``executor_class`` in `.__init__`, or override
+            ``tasks.executor_class`` anywhere in the :ref:`config system
+            <default-values>` (which may allow you to avoid using a custom
+            Program entirely).
 
         .. versionadded:: 1.0
         """
-        executor = self.executor_class(self.collection, self.config, self.core)
+        klass = self.executor_class
+        config_path = self.config.tasks.executor_class
+        if config_path is not None:
+            # TODO: why the heck is this not builtin to importlib?
+            module_path, _, class_name = config_path.rpartition(".")
+            # TODO: worth trying to wrap both of these and raising ImportError
+            # for cases where module exists but class name does not? More
+            # "normal" but also its own possible source of bugs/confusion...
+            module = import_module(module_path)
+            klass = getattr(module, class_name)
+        executor = klass(self.collection, self.config, self.core)
         executor.execute(*self.tasks)
 
     def normalize_argv(self, argv):
@@ -522,6 +597,18 @@ class Program(object):
         """
         return self._name or self.binary.capitalize()
 
+    @property
+    def called_as(self):
+        """
+        Returns the program name we were actually called as.
+
+        Specifically, this is the (Python's os module's concept of a) basename
+        of the first argument in the parsed argument vector.
+
+        .. versionadded:: 1.2
+        """
+        return os.path.basename(self.argv[0])
+
     @property
     def binary(self):
         """
@@ -529,8 +616,18 @@ class Program(object):
 
         .. versionadded:: 1.0
         """
-        return self._binary or os.path.basename(self.argv[0])
+        return self._binary or self.called_as
 
+    @property
+    def binary_names(self):
+        """
+        Derive program's completion-oriented binary name(s) from args & argv.
+
+        .. versionadded:: 1.2
+        """
+        return self._binary_names or [self.called_as]
+
+    # TODO 2.0: ugh rename this or core_args, they are too confusing
     @property
     def args(self):
         """
@@ -611,6 +708,15 @@ class Program(object):
         except CollectionNotFound as e:
             raise Exit("Can't find any collection named {!r}!".format(e.name))
 
+    def _update_core_context(self, context, new_args):
+        # Update core context w/ core_via_task args, if and only if the
+        # via-task version of the arg was truly given a value.
+        # TODO: push this into an Argument-aware Lexicon subclass and
+        # .update()?
+        for key, arg in new_args.items():
+            if arg.got_value:
+                context.args[key]._value = arg._value
+
     def parse_tasks(self):
         """
         Parse leftover args, which are typically tasks & per-task args.
@@ -619,6 +725,10 @@ class Program(object):
         parsed per-task contexts, and ``self.core_via_tasks`` to a context
         holding any core flags seen within the task contexts.
 
+        Also modifies ``self.core`` to include the data from ``core_via_tasks``
+        (so that it correctly reflects any supplied core flags regardless of
+        where they appeared).
+
         .. versionadded:: 1.0
         """
         self.parser = Parser(
@@ -627,8 +737,10 @@ class Program(object):
         )
         debug("Parsing tasks against {!r}".format(self.collection))
         result = self.parser.parse_argv(self.core.unparsed)
-        # TODO: can we easily 'merge' this into self.core? Ehh
         self.core_via_tasks = result.pop(0)
+        self._update_core_context(
+            context=self.core[0], new_args=self.core_via_tasks.args
+        )
         self.tasks = result
         debug("Resulting task contexts: {!r}".format(self.tasks))
 
